/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.api.recipe.recipes;

import com.google.gson.JsonObject;
import reborncore.common.crafting.RebornRecipe;
import reborncore.common.crafting.RebornRecipeType;
import reborncore.common.crafting.ingredient.RebornIngredient;

import java.util.Collections;
import java.util.List;
import net.minecraft.class_1263;
import net.minecraft.class_1715;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1865;
import net.minecraft.class_1869;
import net.minecraft.class_1937;
import net.minecraft.class_2371;
import net.minecraft.class_2540;
import net.minecraft.class_2960;
import net.minecraft.class_3518;

public class RollingMachineRecipe extends RebornRecipe {

	private class_1869 shapedRecipe;

	public RollingMachineRecipe(RebornRecipeType<?> type, class_2960 name) {
		super(type, name);
	}

	public RollingMachineRecipe(RebornRecipeType<?> type, class_2960 name, class_1869 recipe) {
		super(type, name);
		this.shapedRecipe = recipe;
	}

	@Override
	public void deserialize(JsonObject jsonObject) {
		if(jsonObject.has("shaped")) {
			JsonObject json = class_3518.method_15296(jsonObject, "shaped");
			shapedRecipe = class_1865.field_9035.method_8121(method_8114(), json);
		} else {
			//This will be handled by the PacketByteBuf deserialize
		}
	}

	@Override
	public void serialize(class_2540 byteBuf) {
		String s = shapedRecipe.method_8114().toString();
		byteBuf.writeInt(s.length());
		byteBuf.method_10814(s);
		class_1865.field_9035.method_8124(byteBuf, shapedRecipe);
	}


	@Override
	public void deserialize(class_2540 byteBuf) {
		class_2960 identifier = new class_2960(byteBuf.method_10800(byteBuf.readInt()));
		shapedRecipe = class_1865.field_9035.method_8122(identifier, byteBuf);
	}

	@Override
	public class_2371<RebornIngredient> getRebornIngredients() {
		return class_2371.method_10211();
	}

	@Override
	public List<class_1799> getOutputs() {
		return Collections.singletonList(shapedRecipe.method_8110());
	}

	@Override
	public class_1799 method_8110() {
		return shapedRecipe.method_8110();
	}

	@Override
	public class_2371<class_1856> method_8117() {
		return shapedRecipe.method_8117();
	}

	@Override
	public boolean method_8115(class_1263 inv, class_1937 worldIn) {
		return shapedRecipe.method_17728((class_1715) inv, worldIn);
	}

	@Override
	public class_1799 method_8116(class_1263 inv) {
		return shapedRecipe.method_17727((class_1715) inv);
	}

	@Override
	public boolean method_8113(int width, int height) {
		return shapedRecipe.method_8113(width, height);
	}

	public class_1869 getShapedRecipe() {
		return shapedRecipe;
	}
}
