/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.cable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2586;
import net.minecraft.class_2622;
import net.minecraft.class_3000;
import org.apache.commons.lang3.tuple.Pair;
import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.StringUtils;
import team.reborn.energy.Energy;
import team.reborn.energy.EnergySide;
import team.reborn.energy.EnergyStorage;
import team.reborn.energy.EnergyTier;
import techreborn.blocks.cable.CableBlock;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

/**
 * Created by modmuss50 on 19/05/2017.
 */

public class CableBlockEntity extends class_2586
	implements class_3000, IListInfoProvider, IToolDrop, EnergyStorage {
	
	private double energy = 0;
	private TRContent.Cables cableType = null;
	private ArrayList<EnergySide> sendingFace = new ArrayList<>();

	public CableBlockEntity() {
		super(TRBlockEntities.CABLE);
	}

	public CableBlockEntity(TRContent.Cables type) {
		super(TRBlockEntities.CABLE);
		this.cableType = type;
	}

	private TRContent.Cables getCableType() {
		if (cableType != null) {
			return cableType;
		}
		if (field_11863 == null) {
			return TRContent.Cables.COPPER;
		}
		class_2248 block = field_11863.method_8320(field_11867).method_11614();
		if(block instanceof CableBlock){
			return ((CableBlock) block).type;
		}
		//Something has gone wrong if this happens
		return TRContent.Cables.COPPER;
	}

	@Override
    public class_2487 method_16887() {
        return method_11007(new class_2487());
    }

    @Override
    public class_2622 method_16886() {
        class_2487 nbtTag = new class_2487();
        method_11007(nbtTag);
        return new class_2622(method_11016(), 1, nbtTag);
    }


    @Override
    public void method_11014(class_2487 compound) {
        super.method_11014(compound);
        if (compound.method_10545("CableBlockEntity")) {
            energy = compound.method_10562("CableBlockEntity").method_10550("power");
        }
    }

    @Override
    public class_2487 method_11007(class_2487 compound) {
        super.method_11007(compound);
        if (energy > 0) {
        	class_2487 data = new class_2487();
    		compound.method_10566("CableBlockEntity", data);
        }
        return compound;
    }

	@Override
	public void method_16896() {
		if (field_11863.field_9236) {
			return;
		}

		sendingFace.clear();

		if(getEnergy() == 0){
			return;
		}

		ArrayList<Pair<class_2586, class_2350>> acceptors = new ArrayList<>();
		for (class_2350 face : class_2350.values()) {
			EnergySide side = EnergySide.fromMinecraft(face);
			class_2586 blockEntity = field_11863.method_8321(field_11867.method_10093(face));

			if (blockEntity != null && Energy.valid(blockEntity)) {
				if (blockEntity instanceof CableBlockEntity && energy <= Energy.of(blockEntity).side(EnergySide.fromMinecraft(face)).getEnergy()) {
					continue;
				}
				if(Energy.of(blockEntity).side(EnergySide.fromMinecraft(face.method_10153())).getMaxInput() > 0){
					acceptors.add(Pair.of(blockEntity, face));
					if (!sendingFace.contains(side)) {
						sendingFace.add(side);
					}
				}
			}
		}

		if (acceptors.isEmpty()) {
			return;
		}
		Collections.shuffle(acceptors);

        acceptors.forEach(pair -> {
	        Energy.of(this)
		        .into(Energy.of(pair.getLeft()).side(EnergySide.fromMinecraft(pair.getRight().method_10153())))
		        .move();
        });
	}

    // IListInfoProvider
	@Override
	public void addInfo(List<class_2561> info, boolean isReal, boolean hasData) {
			info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.transferRate") + ": "
				+ class_124.field_1065
				+ PowerSystem.getLocaliszedPowerFormatted(getCableType().transferRate) + "/t"));
			info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.tier") + ": "
				+ class_124.field_1065 + StringUtils.toFirstCapitalAllLowercase(getCableType().tier.toString())));
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return new class_1799(getCableType().block);
	}


	public double getEnergy() {
		return getStored(EnergySide.UNKNOWN);
	}

	public void setEnergy(double energy) {
		setStored(energy);
	}

	public double useEnergy(double energyOut, boolean simulate) {
		if (energyOut > energy) {
			energyOut = energy;
		}
		if (!simulate) {
			setEnergy(energy - energyOut);
		}
		return energyOut;
	}

	public boolean canAcceptEnergy(EnergySide direction) {
		if (sendingFace.contains(direction)) {
			return false;
		}
		return getMaxStoredPower() != getEnergy();
	}

	@Override
	public double getMaxInput(EnergySide side) {
		if(!canAcceptEnergy(side)) {
			return 0;
		}
		return getCableType().transferRate;
	}

	@Override
	public double getMaxOutput(EnergySide side) {
		return getCableType().transferRate;
	}

	@Override
	public double getMaxStoredPower() {
		return getCableType().transferRate * 4;
	}

	@Override
	public EnergyTier getTier() {
		return PowerAcceptorBlockEntity.getTier(getCableType().transferRate);
	}

	@Override
	public double getStored(EnergySide face) {
		return energy;
	}

	@Override
	public void setStored(double amount) {
		this.energy = amount;
	}
}
