/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.storage.idsu;

import net.minecraft.class_1657;
import net.minecraft.class_2487;
import org.apache.commons.lang3.StringUtils;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import team.reborn.energy.EnergySide;
import team.reborn.energy.EnergyTier;
import techreborn.blockentity.storage.EnergyStorageBlockEntity;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

public class InterdimensionalSUBlockEntity extends EnergyStorageBlockEntity implements IContainerProvider {

	public String ownerUdid;

	//This is the energy value that is synced to the client
	private double clientEnergy;

	public InterdimensionalSUBlockEntity() {
		super(TRBlockEntities.INTERDIMENSIONAL_SU, "IDSU", 2, TRContent.Machine.INTERDIMENSIONAL_SU.block, EnergyTier.EXTREME, TechRebornConfig.idsuMaxInput, TechRebornConfig.idsuMaxOutput, TechRebornConfig.idsuMaxEnergy);
	}

	@Override
	public double getStored(EnergySide face) {
		if (ownerUdid == null || ownerUdid.isEmpty()) {
			return 0.0;
		}
		if(field_11863.field_9236){
			return clientEnergy;
		}
		return IDSUManager.getPlayer(field_11863, ownerUdid).getEnergy();
	}

	@Override
	public void setStored(double energy) {
		if (ownerUdid == null || ownerUdid.isEmpty()) {
			return;
		}
		if(field_11863.field_9236){
			clientEnergy = energy;
		} else {
			IDSUManager.getPlayer(field_11863, ownerUdid).setEnergy(energy);
		}
	}
	
	@Override
	public double useEnergy(double extract, boolean simulate) {
		if (ownerUdid == null || ownerUdid.isEmpty()) {
			return 0.0;
		}
		if(field_11863.field_9236){
			throw new UnsupportedOperationException("cannot set energy on the client!");
		}
		double energy = IDSUManager.getPlayer(field_11863, ownerUdid).getEnergy();
		if (extract > energy) {
			extract = energy;
		}
		if (!simulate) {
			setEnergy(energy - extract);
		}
		return extract;
	}
	
	@Override
	public boolean canUseEnergy(double input) {
		if (ownerUdid == null || ownerUdid.isEmpty()) {
			return false;
		}
		if(field_11863.field_9236){
			throw new UnsupportedOperationException("cannot set energy on the client!");
		}
		return input <= IDSUManager.getPlayer(field_11863, ownerUdid).getEnergy();
	}

	@Override
	public void method_11014(class_2487 nbttagcompound) {
		super.method_11014(nbttagcompound);
		this.ownerUdid = nbttagcompound.method_10558("ownerUdid");
	}

	@Override
	public class_2487 method_11007(class_2487 nbttagcompound) {
		super.method_11007(nbttagcompound);
		if (ownerUdid == null || StringUtils.isEmpty(ownerUdid)) {
			return nbttagcompound;
		}
		nbttagcompound.method_10582("ownerUdid", this.ownerUdid);
		return nbttagcompound;
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("idsu").player(player.field_7514).inventory().hotbar().armor()
			.complete(8, 18).addArmor().addInventory().blockEntity(this).energySlot(0, 62, 45).energySlot(1, 98, 45)
			.syncEnergyValue().addInventory().create(this, syncID);
	}

	@Override
	public boolean shouldHanldeEnergyNBT() {
		return false;
	}
}
