/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.gui;

import net.minecraft.class_1657;
import reborncore.client.ClientChunkManager;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.gui.builder.GuiBase;
import reborncore.client.gui.builder.widget.GuiButtonSimple;
import reborncore.client.gui.builder.widget.GuiButtonUpDown;
import reborncore.client.gui.builder.widget.GuiButtonUpDown.UpDownButtonType;
import reborncore.common.network.NetworkManager;
import techreborn.blockentity.machine.tier3.ChunkLoaderBlockEntity;
import techreborn.packets.ServerboundPackets;

public class GuiChunkLoader extends GuiBase<BuiltContainer> {

	ChunkLoaderBlockEntity blockEntity;

	public GuiChunkLoader(int syncID, class_1657 player, ChunkLoaderBlockEntity blockEntity) {
		super(player, blockEntity, blockEntity.createContainer(syncID, player));
		this.blockEntity = blockEntity;
	}
	
	public void init() {
		super.init();
		addButton(new GuiButtonUpDown(field_2776 + 64, field_2800 + 40, this, b -> onClick(5), UpDownButtonType.FASTFORWARD));
		addButton(new GuiButtonUpDown(field_2776 + 64 + 12, field_2800 + 40, this, b -> onClick(1), UpDownButtonType.FORWARD));
		addButton(new GuiButtonUpDown(field_2776 + 64 + 24, field_2800 + 40, this, b -> onClick(-1), UpDownButtonType.REWIND));
		addButton(new GuiButtonUpDown(field_2776 + 64 + 36, field_2800 + 40, this, b -> onClick(-5), UpDownButtonType.FASTREWIND));

		addButton(new GuiButtonSimple(field_2776 + 10, field_2800 + 70, 155, 20, "Toggle Loaded Chunks", b -> ClientChunkManager.toggleLoadedChunks(blockEntity.method_11016())));
	}
	
	@Override
	protected void method_2389(float partialTicks, int mouseX, int mouseY) {
		super.method_2389(partialTicks, mouseX, mouseY);
		final Layer layer = Layer.BACKGROUND;

		if (GuiBase.slotConfigType != GuiBase.SlotConfigType.NONE) {
			return;
		}
		String text = "Radius: " + blockEntity.getRadius();
		drawCentredString(text, 25, 4210752, layer);
	}

	public void onClick(int amount){
		NetworkManager.sendToServer(ServerboundPackets.createPacketChunkloader(amount, blockEntity, ClientChunkManager.hasChunksForLoader(blockEntity.method_11016())));
	}	
}
